searchNodes=[{"doc":"","ref":"Definject.html","title":"Definject","type":"module"},{"doc":"definject transforms a function to accept a map where dependent functions can be injected. import Definject definject send_welcome_email(user_id) do %{email: email} = Repo.get(User, user_id) welcome_email(to: email) |&gt; Mailer.send() end is expanded into def send_welcome_email(user_id, %{} = deps \\\\ %{}) do %{email: email} = (deps[&amp;Repo.get/2] || &amp;Repo.get/2).(User, user_id) welcome_email(to: email) |&gt; (deps[&amp;Mailer.send/1] || &amp;Mailer.send/1).() end Note that local function calls like welcome_email(to: email) are not expanded unless it is prepended with __MODULE__. Now, you can inject mock functions in tests. test &quot;send_welcome_email&quot; do Accounts.send_welcome_email(100, %{ &amp;Repo.get/2 =&gt; fn User, 100 -&gt; %User{email: &quot;mr.jechol@gmail.com&quot;} end, &amp;Mailer.send/1 =&gt; fn %Email{to: &quot;mr.jechol@gmail.com&quot;, subject: &quot;Welcome&quot;} -&gt; Process.send(self(), :email_sent) end }) assert_receive :email_sent end definject raises if the passed map includes a function that&#39;s not called within the injected function. You can disable this by adding strict: false option. test &quot;send_welcome_email with strict: false&quot; do Accounts.send_welcome_email(100, %{ &amp;Repo.get/2 =&gt; fn User, 100 -&gt; %User{email: &quot;mr.jechol@gmail.com&quot;} end, &amp;Repo.all/1 =&gt; fn _ -&gt; [%User{email: &quot;mr.jechol@gmail.com&quot;}] end, # Unused strict: false, }) end","ref":"Definject.html#definject/2","title":"Definject.definject/2","type":"macro"},{"doc":"If you don&#39;t need pattern matching in mock function, mock/1 can be used to reduce boilerplates. test &quot;send_welcome_email with mock/1&quot; do Accounts.send_welcome_email( 100, mock(%{ &amp;Repo.get/2 =&gt; %User{email: &quot;mr.jechol@gmail.com&quot;}, &amp;Mailer.send/1 =&gt; Process.send(self(), :email_sent) }) ) assert_receive :email_sent end Note that Process.send(self(), :email_sent) is surrounded by fn _ -&gt; end when expanded.","ref":"Definject.html#mock/1","title":"Definject.mock/1","type":"macro"},{"doc":"Unobtrusive Dependency Injector for Elixir","ref":"readme.html","title":"README","type":"extras"},{"doc":"Let&#39;s say we want to test following function with mocks for Repo and Mailer. def send_welcome_email(user_id) do %{email: email} = Repo.get(User, user_id) welcome_email(to: email) |&gt; Mailer.send() end Here&#39;s how you use one of the existing mock libraries: def send_welcome_email(user_id, repo \\\\ Repo, mailer \\\\ Mailer) do %{email: email} = repo.get(User, user_id) welcome_email(to: email) |&gt; mailer.send() end First, I believe that this approach is too obtrusive as it requires modifying the function body to make it testable. Second, with Repo replaced with repo, the compiler can no longer guarantee the existence of Repo.get/2 function. definject does not require you to modify function arguments or body. Instead, you just need to replace def with definject. It also allows injecting different mocks to each function. It also does not limit using :async option as mocks are contained in each test function.","ref":"readme.html#why","title":"README - Why?","type":"extras"},{"doc":"The package can be installed by adding definject to your list of dependencies in mix.exs: def deps do [{:definject, &quot;~&gt; 0.6.1&quot;}] end By default, definject is replaced with def in all but the test environment. Add the below configuration to enable in other environments. config :definject, :enable, true","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"definject definject transforms a function to accept a map where dependent functions can be injected. import Definject definject send_welcome_email(user_id) do %{email: email} = Repo.get(User, user_id) welcome_email(to: email) |&gt; Mailer.send() end is expanded into def send_welcome_email(user_id, %{} = deps \\\\ %{}) do %{email: email} = (deps[&amp;Repo.get/2] || &amp;Repo.get/2).(User, user_id) welcome_email(to: email) |&gt; (deps[&amp;Mailer.send/1] || &amp;Mailer.send/1).() end Note that local function calls like welcome_email(to: email) are not expanded unless it is prepended with __MODULE__. Now, you can inject mock functions in tests. test &quot;send_welcome_email&quot; do Accounts.send_welcome_email(100, %{ &amp;Repo.get/2 =&gt; fn User, 100 -&gt; %User{email: &quot;mr.jechol@gmail.com&quot;} end, &amp;Mailer.send/1 =&gt; fn %Email{to: &quot;mr.jechol@gmail.com&quot;, subject: &quot;Welcome&quot;} -&gt; Process.send(self(), :email_sent) end }) assert_receive :email_sent end definject raises if the passed map includes a function that&#39;s not called within the injected function. You can disable this by adding strict: false option. test &quot;send_welcome_email with strict: false&quot; do Accounts.send_welcome_email(100, %{ &amp;Repo.get/2 =&gt; fn User, 100 -&gt; %User{email: &quot;mr.jechol@gmail.com&quot;} end, &amp;Repo.all/1 =&gt; fn _ -&gt; [%User{email: &quot;mr.jechol@gmail.com&quot;}] end, # Unused strict: false, }) end mock If you don&#39;t need pattern matching in mock function, mock/1 can be used to reduce boilerplates. test &quot;send_welcome_email with mock/1&quot; do Accounts.send_welcome_email( 100, mock(%{ &amp;Repo.get/2 =&gt; %User{email: &quot;mr.jechol@gmail.com&quot;}, &amp;Mailer.send/1 =&gt; Process.send(self(), :email_sent) }) ) assert_receive :email_sent end Note that Process.send(self(), :email_sent) is surrounded by fn _ -&gt; end when expanded.","ref":"readme.html#usage","title":"README - Usage","type":"extras"},{"doc":"This project is licensed under the MIT License - see the LICENSE file for details","ref":"readme.html#license","title":"README - License","type":"extras"}]